/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "support/PortableMacros.h"
#include "syslog/SysLog.h"
#include "queue/PortableQueue.h"

static const char * const _logLevelNames[] = {
   "None",
   "Critical Errors Only",
   "Errors Only",
   "Warnings and Errors Only",
   "Informational",
   "Debug",
   "Trace"
};

static const char * const _logLevelKeywords[] = {
   "none",
   "critical",
   "errors",
   "warnings",
   "info",
   "debug",
   "trace"
};

class DefaultLogger : public LogCallback
{
public:
   DefaultLogger()
      : _fileLogLevel(MUSCLE_LOG_NONE), _consoleLogLevel(MUSCLE_LOG_INFO), _logFile(NULL)
   {
       // empty
   }

   virtual void Log(time_t when, int ll, const char * fmt, va_list argList)
   {
      if ((ll <= _fileLogLevel)&&(_logFile == NULL))
      {                                          
         struct tm * now = localtime(&when);
         char tb[100];                           
         sprintf(tb, "%02i%02i%02i%02i.log", now->tm_mon+1, now->tm_mday, now->tm_hour, now->tm_min); 
         _logFile = fopen(tb, "w");              
         if (_logFile) LogTime(MUSCLE_LOG_INFO, "Created log file %s\n", tb);
      }                                          
      if (ll <= _consoleLogLevel) vprintf(fmt, argList);                  
      if ((ll <= _fileLogLevel)&&(_logFile))
      {                                         
         vfprintf(_logFile, fmt, argList);       
         fflush(_logFile);                       
      }                                          
   }

   int _fileLogLevel;
   int _consoleLogLevel;
   FILE * _logFile;
};

static PortableQueue<LogCallback *> _logCallbacks;
static DefaultLogger _dl;

const char * GetLogLevelName(int ll)
{
   return ((ll>=0)&&(ll<(int) ARRAYITEMS(_logLevelNames))) ? _logLevelNames[ll] : "???";
}

int ParseLogLevelKeyword(const char * keyword)
{
   for (size_t i=0; i<ARRAYITEMS(_logLevelKeywords); i++) if (strcmp(keyword, _logLevelKeywords[i]) == 0) return i;
   return -1;
}

int GetFileLogLevel()
{
   return _dl._fileLogLevel;
}

int GetConsoleLogLevel()
{
   return _dl._consoleLogLevel;
}

int GetMaxLogLevel()
{
   return (_dl._consoleLogLevel > _dl._fileLogLevel) ? _dl._consoleLogLevel : _dl._fileLogLevel;
}

void SetFileLogLevel(int loglevel)
{
   _dl._fileLogLevel = loglevel;
   LogTime(MUSCLE_LOG_INFO, "File logging level set to: %s\n", GetLogLevelName(_dl._fileLogLevel));
}

void SetConsoleLogLevel(int loglevel)
{
   _dl._consoleLogLevel = loglevel;
   LogTime(MUSCLE_LOG_INFO, "Console logging level set to: %s\n", GetLogLevelName(_dl._consoleLogLevel));
}

#define DO_LOGGING                                    \
{                                                     \
   time_t when = time(NULL);                          \
   va_list argList;                                   \
   va_start(argList, fmt);                            \
   _dl.Log(when, ll, fmt, argList);                   \
   va_end(argList);                                   \
   int numLogCallbacks = _logCallbacks.CountItems();  \
   for (int i=0; i<numLogCallbacks; i++)              \
   {                                                  \
      LogCallback * next = _logCallbacks[i];          \
      va_start(argList, fmt);                         \
      next->Log(when, ll, fmt, argList);              \
      va_end(argList);                                \
   }                                                  \
}

void LogTime(int ll, const char * fmt, ...)
{
   time_t n = time(NULL);
   struct tm * now = localtime(&n);

   // Record time stamp to our file/stdout logs only
   char buf[128];
   sprintf(buf, "[%c %02i/%02i %02i:%02i:%02i] ", GetLogLevelName(ll)[0], now->tm_mon+1, now->tm_mday, now->tm_hour, now->tm_min, now->tm_sec);
   va_list dummyList;
   va_start(dummyList, fmt);  // not used
   _dl.Log(n, ll, buf, dummyList);
   va_end(dummyList);
   
   // Log message to file/stdio and callbacks
   DO_LOGGING;
}

void Log(int ll, const char * fmt, ...)
{
   DO_LOGGING;
}

void AddLogCallback(LogCallback * cb)
{
   _logCallbacks.AddTail(cb);
}

void RemoveLogCallback(LogCallback * cb, bool deleteToo)
{
   for (int i=_logCallbacks.CountItems()-1; i>=0; i--)
   {
      LogCallback * next = _logCallbacks[i];
      if ((next == cb)||(cb == NULL))
      {
         _logCallbacks.RemoveItemAt(i);
         if (deleteToo) delete next;
         if (cb) break;
      }
   }
}
